/*
Copyright (c) 2001 David Drysdale

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the author shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the author.
*/


import java.net.InetAddress;
import java.util.Date;

class FontDescription {
    public String fontName;
    public Date createDate;
    public String creatorString;
    public String[] letterString;
    public String font = null;

    // Default constructor
    FontDescription() {
        letterString = new String[26];
        fontName = new String("");
        createDate = new Date();
        // try to figure out local creator
        creatorString = new String("unknown");
        InetAddress where = null;

        try {
            where = InetAddress.getLocalHost();
        } catch (java.net.UnknownHostException t) {}
        if (where != null) creatorString = new String("someone@" + where.getHostName());
    }

    // Construct from a given text description string
    FontDescription(String fontString) {
        // break the supplied font description into its bits
        letterString = new String[26];
    
        // Prefix the font description with a "\n" so we do not have
        // to do a special case below (*)
        font = "\n" + fontString;

        // find the font name "font : <name>\n"
        fontName = findField(font, "font", ':');
        if (fontName == null) fontName = new String("");

        // find the creation date "create date : <date>\n"
        String cd = findField(font, "create date", ':');
        if (cd == null) {
            createDate = new Date();
        } else {
            try {
                createDate = new Date(Date.parse(cd));
            } catch (IllegalArgumentException e) {
                createDate = new Date();
            }
        }

        // find the creator "creator : <name>\n"
        creatorString = findField(font, "creator", ':');
        if (creatorString == null) creatorString = new String("");

        for (char letter = 'a'; letter <= 'z'; letter++) {
            // try to find this letter in the passed font string.  It
            // will look like "\nx 01627abe37189900\n" (*)
            String codeString = findLetter(font, new String("\n" + letter + ' '));
                
            // if we found it, set the letter up.  Otherwise, clear it
            if (codeString != null) {
                letterString[letter - 'a'] = codeString;
            } else {
                letterString[letter - 'a'] = new String("");
            }
        }
    }

    // Utility function to find the value of a named field in a text line
    // We look for:
    //  - the first instance of the label
    //  - followed by 0 or more whitespace
    //  - followed by the separator 
    //  - followed by 0 or more whitespace
    //  - followed by the answer
    //  - followed by 0 or more whitespace
    //  - followed by newline
    // In regexp terms, this is the same as running the following on text
    //  s/.*label[ \t]*sep[ \t]*\(.*\)\n/\1/     (then string trailing space)
    // If label is not found, return null
    String findField(String text, String label, char sep) {
        int offset = text.indexOf(label, 0);
        if (offset == -1) return null;

        offset += label.length();
        while ((text.charAt(offset) != sep) && (offset < text.length()) &&
               Character.isSpace(text.charAt(offset))) {
            offset++;
        }
        if (offset >= text.length()) return null; // ran off end of string
        // move past separator
        offset++;
        // skip whitespace again
        while ((text.charAt(offset) != '\n') && (offset < text.length()) &&
               Character.isSpace(text.charAt(offset))) {
            offset++;
        }
        if (offset >= text.length()) return null; // ran off end of string
        if (text.charAt(offset) == '\n') return new String(""); // empty answer
        int start = offset;
        int lastnonblank = offset;
        // look for end of line
        while ((text.charAt(offset) != '\n') && (offset < text.length())) {
            if (!Character.isSpace(text.charAt(offset))) {
                lastnonblank = offset;
            }
            offset++;
        }
        // offset is now either '\n' or past end of base string.  So the string
        // we want to return starts at <start> and runs to <lastnonblank>
        return text.substring(start, lastnonblank+1);
    }

    // Utility function to find the value of a named field in a text line
    // We look for:
    //  - the first instance of the label
    //  - followed by 0 or more whitespace
    //  - followed by the answer
    //  - followed by 0 or more whitespace
    //  - followed by newline
    // In regexp terms, this is the same as running the following on text
    //  s/.*label[ \t]*\(.*\)\n/\1/     (then string trailing space)
    // If label is not found, return null
    String findLetter(String text, String label) {
        int offset = text.indexOf(label, 0);
        if (offset == -1) return null;

        offset += label.length();
        // skip whitespace
        while ((text.charAt(offset) != '\n') && (offset < text.length()) &&
               Character.isSpace(text.charAt(offset))) {
            offset++;
        }
        if (offset >= text.length()) return null; // ran off end of string
        if (text.charAt(offset) == '\n') return new String(""); // empty answer
        int start = offset;
        int lastnonblank = offset;
        // look for end of line
        while ((text.charAt(offset) != '\n') && (offset < text.length())) {
            if (!Character.isSpace(text.charAt(offset))) {
                lastnonblank = offset;
            }
            offset++;
        }
        // offset is now either '\n' or past end of base string.  So the string
        // we want to return starts at <start> and runs to <lastnonblank>
        return text.substring(start, lastnonblank+1);
    }
}
